@@ -2,7 +2,7 @@
 The pycity_scheduling framework
 
 
-Copyright (C) 2022,
+Copyright (C) 2023,
 Institute for Automation of Complex Power Systems (ACS),
 E.ON Energy Research Center (E.ON ERC),
 RWTH Aachen University
@@ -24,12 +24,13 @@ OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 
 import numpy as np
 import time
+import warnings
 import pyomo.environ as pyomo
 from pyomo.solvers.plugins.solvers.persistent_solver import PersistentSolver
 from pyomo.opt import SolverStatus, TerminationCondition
 
 from pycity_scheduling.classes import CityDistrict, Building
-from pycity_scheduling.exceptions import NonoptimalError, MaxIterationError
+from pycity_scheduling.exceptions import NonoptimalError
 from pycity_scheduling.solvers import DEFAULT_SOLVER, DEFAULT_SOLVER_OPTIONS
 
 
@@ -210,7 +211,7 @@ class IterationAlgorithm(OptimizationAlgorithm):
             iterations.append((iterations[-1] + 1) if len(iterations) > 0 else 1)
             self._iteration(results, params, debug=debug)
             self._save_time(results, params)
-            is_last_iteration = self._is_last_iteration(results, params)
+            is_last_iteration = self._is_last_iteration(results, params, debug)
         return
 
     def _iteration(self, results, params, debug):
@@ -222,29 +223,35 @@ class IterationAlgorithm(OptimizationAlgorithm):
             Dictionary in which performance values of the algorithm are stored.
         params : dict
             Dictionary in which intermediate results are stored.
+        """
+        return
+
+    def _is_last_iteration(self, results, params, debug):
+        """Returns True if the current iteration is the last one.
+            It checks if the iteration limit is exceeded.
+            Overwrite this function to apply additional, individual stopping criteria other than the iteration limit.
+            Anyway, this parent method must be called in a child method, so that the check of an exceeded iteration
+            limit below is executed.
+
+        Parameters
+        ----------
         debug : bool, optional
             Specify whether detailed debug information shall be printed.
         Raises
         ------
-        MaxIterationError
+        Warning
             If the stopping criteria can not be reached in max_iterations.
         NonoptimalError
             If no feasible solution for the city district is found or a solver
             problem is encountered.
         """
-        if results["iterations"][-1] > self.max_iterations:
+        if results["iterations"][-1] >= self.max_iterations:
             if debug:
-                print(
-                    "Exceeded iteration limit of {0} iterations. "
-                    "Norms are ||r|| =  {1}, ||s|| = {2}."
-                        .format(self.max_iterations, results["r_norms"][-1], results["s_norms"][-1])
-                )
-            raise MaxIterationError("Iteration Limit exceeded.")
-        return
-
-    def _is_last_iteration(self, results, params):
-        """Returns True if the current iteration is the last one."""
-        raise NotImplementedError("This method should be implemented by subclass.")
+                warnings.warn("User defined iteration limit exceeded")
+            print("Exceeded the user defined iteration limit of {0} iterations. "
+                  "Terminating the iterative algorithm.".format(self.max_iterations))
+            return True
+        return False
 
 
 class DistributedAlgorithm(OptimizationAlgorithm):
@@ -331,12 +338,13 @@ class SolverNode:
         protected from deviations. Second entry defines the magnitude of
         deviations which are considered.
     """
-    def __init__(self, solver, solver_options, entities, mode="convex", robustness=None):
+    def __init__(self, solver, solver_options, entities, mode="convex", robustness=None, write_ilp_file=False):
         self.solver = pyomo.SolverFactory(solver, node_ids=[entity.id for entity in entities],
                                           **solver_options.get("__call__", {}))
         self.solver_options = solver_options
         self.is_persistent = isinstance(self.solver, PersistentSolver)
         self.robustness = robustness
+        self.write_ilp_file = write_ilp_file
         self.entities = entities
         self.mode = mode
         self.model = None
@@ -377,6 +385,14 @@ class SolverNode:
             self.solver.set_instance(self.model, **self.solver_options.get("set_instance", {}))
         return
 
+    def constr_update(self):
+        """Only propagate the constraints update of the model."""
+        if self.is_persistent:
+            self.solver.set_instance(self.model, **self.solver_options.get("set_instance", {}))
+        else:
+            pass
+        return
+
     def obj_update(self):
         """Only propagate the objective value update of the model."""
         if self.is_persistent:
@@ -412,7 +428,7 @@ class SolverNode:
             result = self.solver.solve(self.model, **solve_options)
         if result.solver.termination_condition != TerminationCondition.optimal or \
                 result.solver.status != SolverStatus.ok:
-            if debug:
+            if debug and self.write_ilp_file:
                 import pycity_scheduling.util.debug as debug
                 debug.analyze_model(self.model, self.solver, result)
             raise NonoptimalError("Could not retrieve schedule from model.")
