@@ -2,7 +2,7 @@
 The pycity_scheduling framework
 
 
-Copyright (C) 2022,
+Copyright (C) 2023,
 Institute for Automation of Complex Power Systems (ACS),
 E.ON Energy Research Center (E.ON ERC),
 RWTH Aachen University
@@ -58,29 +58,30 @@ class TestConstraints(unittest.TestCase):
         ee = ElectricalEntity(self.env)
         m = pyomo.ConcreteModel()
         ee.populate_model(m, "integer")
-        o = pyomo.sum_product(ee.model.p_el_vars, ee.model.p_el_vars)
-        o -= 2 * 0.3 * pyomo.sum_product(ee.model.p_el_vars)
+        o = 0
+        for t in range(len(ee.model.p_el_vars)):
+            o += ee.model.p_el_vars[t] * ee.model.p_el_vars[t]
+        o -= 2 * 0.3 * sum(ee.model.p_el_vars[t] for t in range(len(ee.model.p_el_vars)))
         m.o = pyomo.Objective(expr=o)
         self.assertEqual(_get_constr_count(ee.model), 0)
         lal_constr = gen_constrs.LowerActivationLimit(ee, "p_el", 0.5, 1)
 
         lal_constr.apply(ee.model, "convex")
         self.assertEqual(_get_constr_count(ee.model), 0)
-        opt = pyomo.SolverFactory(solvers.DEFAULT_SOLVER)
-        opt.solve(m)
+        solve_model(m)
         ee.update_schedule()
         for i in ee.op_time_vec:
-            self.assertAlmostEqual(ee.p_el_schedule[i], 0.3, 4)
-            self.assertAlmostEqual(ee.p_el_state_schedule[i], 1.0, 4)
+            self.assertAlmostEqual(ee.p_el_schedule[i], 0.3, 3)
+            self.assertAlmostEqual(ee.p_el_state_schedule[i], 1.0, 3)
 
         lal_constr.apply(ee.model, "integer")
         self.assertEqual(_get_constr_count(ee.model), 12*2)
 
-        opt.solve(m)
+        solve_model(m)
         ee.update_schedule()
         for i in ee.op_time_vec:
-            self.assertAlmostEqual(ee.p_el_schedule[i], 0.5, 4)
-            self.assertAlmostEqual(ee.p_el_state_schedule[i], 1.0, 4)
+            self.assertAlmostEqual(ee.p_el_schedule[i], 0.5, 3)
+            self.assertAlmostEqual(ee.p_el_state_schedule[i], 1.0, 3)
         return
 
 
@@ -317,11 +318,10 @@ class TestSubpackage(unittest.TestCase):
             util.extract_pyomo_values(v)
 
         m = pyomo.ConcreteModel()
-        opt = pyomo.SolverFactory(solvers.DEFAULT_SOLVER)
 
         m.v = pyomo.Var(domain=pyomo.Reals)
         m.c = pyomo.Constraint(expr=m.v == 1.0)
-        opt.solve(m)
+        solve_model(m)
         e = util.extract_pyomo_values(m.v)
         self.assertEqual(1, e)
         self.assertIs(float, type(e))
@@ -330,7 +330,7 @@ class TestSubpackage(unittest.TestCase):
         m.a = pyomo.Var(domain=pyomo.Reals)
         m.c = pyomo.Constraint(expr=m.a == 1.0)
         m.v = pyomo.Var(pyomo.RangeSet(1, 2), domain=pyomo.Reals)
-        opt.solve(m)
+        solve_model(m)
 
         m.v[1].value = 1
         m.v[2].value = 2
@@ -343,7 +343,7 @@ class TestSubpackage(unittest.TestCase):
         m.a = pyomo.Var(domain=pyomo.Reals)
         m.c = pyomo.Constraint(expr=m.a == 1.0)
         m.v = pyomo.Var(domain=pyomo.Integers, bounds=(4, 5))
-        opt.solve(m)
+        solve_model(m)
         e = util.extract_pyomo_values(m.v)
         self.assertEqual(4, e)
 
@@ -351,7 +351,7 @@ class TestSubpackage(unittest.TestCase):
         m.a = pyomo.Var(domain=pyomo.Reals)
         m.c = pyomo.Constraint(expr=m.a == 1.0)
         m.v = pyomo.Var(domain=pyomo.Integers, bounds=(4.1, 4.3))
-        opt.solve(m)
+        solve_model(m)
         with self.assertRaises(SchedulingError):
             util.extract_pyomo_values(m.v)
 
@@ -359,7 +359,7 @@ class TestSubpackage(unittest.TestCase):
         m.a = pyomo.Var(domain=pyomo.Reals)
         m.c = pyomo.Constraint(expr=m.a == 1.0)
         m.v = pyomo.Var(pyomo.RangeSet(1, 2), domain=pyomo.Binary, bounds=(0.1, 0.9))
-        opt.solve(m)
+        solve_model(m)
         with self.assertRaises(SchedulingError):
             util.extract_pyomo_values(m.v)
 
@@ -367,7 +367,7 @@ class TestSubpackage(unittest.TestCase):
         m.a = pyomo.Var(domain=pyomo.Reals)
         m.c = pyomo.Constraint(expr=m.a == 1.0)
         m.v = pyomo.Var(domain=pyomo.Integers, bounds=(3.9, 4.3))
-        opt.solve(m)
+        solve_model(m)
         e = util.extract_pyomo_values(m.v)
         self.assertEqual(4, e)
         self.assertIs(int, type(e))
@@ -376,7 +376,7 @@ class TestSubpackage(unittest.TestCase):
         m.a = pyomo.Var(domain=pyomo.Reals)
         m.c = pyomo.Constraint(expr=m.a == 1.0)
         m.v = pyomo.Var(domain=pyomo.Integers)
-        opt.solve(m)
+        solve_model(m)
         e = util.extract_pyomo_values(m.v)
         self.assertEqual(0, e)
         self.assertIs(int, type(e))
@@ -385,7 +385,7 @@ class TestSubpackage(unittest.TestCase):
         m.a = pyomo.Var(domain=pyomo.Reals)
         m.c = pyomo.Constraint(expr=m.a == 1.0)
         m.v = pyomo.Var(domain=pyomo.Integers, bounds=(None, -5))
-        opt.solve(m)
+        solve_model(m)
         e = util.extract_pyomo_values(m.v)
         self.assertEqual(-5, e)
         self.assertIs(int, type(e))
@@ -394,7 +394,7 @@ class TestSubpackage(unittest.TestCase):
         m.a = pyomo.Var(domain=pyomo.Reals)
         m.c = pyomo.Constraint(expr=m.a == 1.0)
         m.v = pyomo.Var(domain=pyomo.Integers, bounds=(-10, -2))
-        opt.solve(m)
+        solve_model(m)
         e = util.extract_pyomo_values(m.v)
         self.assertEqual(-2, e)
         self.assertIs(int, type(e))
@@ -403,7 +403,7 @@ class TestSubpackage(unittest.TestCase):
         m.a = pyomo.Var(domain=pyomo.Reals)
         m.c = pyomo.Constraint(expr=m.a == 1.0)
         m.v = pyomo.Var(domain=pyomo.Integers, bounds=(2, 10))
-        opt.solve(m)
+        solve_model(m)
         e = util.extract_pyomo_values(m.v)
         self.assertEqual(2, e)
         self.assertIs(int, type(e))
@@ -412,7 +412,7 @@ class TestSubpackage(unittest.TestCase):
         m.a = pyomo.Var(domain=pyomo.Reals)
         m.c = pyomo.Constraint(expr=m.a == 1.0)
         m.v = pyomo.Var(pyomo.RangeSet(1, 2), domain=pyomo.Binary)
-        opt.solve(m)
+        solve_model(m)
         e = util.extract_pyomo_values(m.v)
         self.assertEqual(0, e[0])
         self.assertEqual(0, e[1])
@@ -422,7 +422,7 @@ class TestSubpackage(unittest.TestCase):
         m.a = pyomo.Var(domain=pyomo.Reals)
         m.c = pyomo.Constraint(expr=m.a == 1.0)
         m.v = pyomo.Var(pyomo.RangeSet(1, 2), domain=pyomo.Binary, bounds=(1, None))
-        opt.solve(m)
+        solve_model(m)
         e = util.extract_pyomo_values(m.v)
         self.assertEqual(1, e[0])
         self.assertEqual(1, e[1])
@@ -446,3 +446,12 @@ class TimerStub:
         if not from_init:
             raise ValueError
         return 12
+
+
+def solve_model(model):
+    # hack to suppress pyomo no constraint warning
+    if not hasattr(model, "simple_var"):
+        model.simple_var = pyomo.Var(domain=pyomo.Reals, bounds=(None, None), initialize=0)
+        model.simple_constr = pyomo.Constraint(expr=model.simple_var == 1)
+    opt = pyomo.SolverFactory(solvers.DEFAULT_SOLVER)
+    return opt.solve(model, **solvers.DEFAULT_SOLVER_OPTIONS.get("solve", {}))
\ No newline at end of file
